package Autodesk.ADN.Material;

import java.lang.reflect.Type;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

//////////////////////////////////////////////////////////////////////////////////////////
// The main Activity
//
//////////////////////////////////////////////////////////////////////////////////////////
public class AdnMaterialActivity 
	extends Activity
	implements WebServiceConnector.WebServiceResponseListener
{
	//////////////////////////////////////////////////////////////////////////////////////
	// Members
	//
	//////////////////////////////////////////////////////////////////////////////////////
	Gson _parser;
	
	Activity _activity;
		
	private WebServiceConnector _webSrvConnector;
	
	private static final String URL = "http://adnmaterial.cloudapp.net/MaterialSrv.svc/rest/";
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	private class AdnMaterialsItemAdapter 
		extends ArrayAdapter<AdnMaterial>
	{ 	  
		int _resource; 
		Context _context; 
		AdnMaterial _data[] = null;
	
		//////////////////////////////////////////////////////////////////////////////////
		//
		//
		//////////////////////////////////////////////////////////////////////////////////
		public AdnMaterialsItemAdapter(
			Context context, 
			int resource,  
			AdnMaterial data[]) 
		{ 
			super(context, resource, data); 
			
			_resource = resource; 
			
			_context = context;
			
			_data = data;
		} 
		
		//////////////////////////////////////////////////////////////////////////////////
		//
		//
		//////////////////////////////////////////////////////////////////////////////////
		@Override
		public View getView(int position, View convertView, ViewGroup parent) 
		{
			View row = convertView;
			
			TagHolder holder;
			
			if(row == null)
			{
				LayoutInflater inflater = ((Activity)_context).getLayoutInflater();
				
				row = inflater.inflate(_resource, parent, false);
				
				holder = new TagHolder();
				
				holder.Image = (ImageView)row.findViewById(R.id.Image);
				holder.Text = (TextView)row.findViewById(R.id.Text);
				
				row.setTag(holder);
			}
			else
			{
				holder = (TagHolder)row.getTag();
			}
			
			AdnMaterial data = _data[position];
			
			holder.Text.setText(data.Name);
			holder.Image.setImageResource(R.drawable.material);
			
			return row;
		}
		
		class TagHolder
		{
			ImageView Image;
			TextView Text;
		}
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	// Activity creation
	//
	//////////////////////////////////////////////////////////////////////////////////////
	@Override
	protected void onCreate(Bundle savedInstanceState) 
	{
		super.onCreate(savedInstanceState);
		
		_activity = this;
		
		_parser = new Gson();
		
		setTitle("ADN Material Demo");
		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
		
		setResult(Activity.RESULT_CANCELED);
		setContentView(R.layout.materials_listview);
		
		_webSrvConnector = new WebServiceConnector(
			URL, 
			"Materials",
			this);	
		
		View header = (View)getLayoutInflater().inflate(
				R.layout.materials_listview_header, null);
		
		ListView lv = (ListView) findViewById(R.id.materialsList);
		
		lv.addHeaderView(header, null, false);
		
		Refresh();
	}
	
	private void Refresh()
	{
		_webSrvConnector.NewRequest();
		_webSrvConnector.InvokeWebGet();
		
		setTitle("ADN Material Demo [Connecting Web Service...]");
		setProgressBarIndeterminateVisibility(true);
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	@Override
    public void onConfigurationChanged(Configuration newConfig) 
    {
        super.onConfigurationChanged(newConfig);
    }
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	@Override
	protected void onDestroy() 
	{
		super.onDestroy();
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	@Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        MenuInflater inflater = getMenuInflater();
        
        inflater.inflate(R.menu.options, menu);
        
        return true;
    }
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) 
	{
		switch (item.getItemId()) 
		{
			case R.id.menu_refresh:
				
				Refresh();
				return true;
		}
		
		return false;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	private OnItemClickListener _itemClickListener = new OnItemClickListener() 
	{
		public void onItemClick(AdapterView<?> parent, View v, int position, long id) 
		{
			Object item = parent.getItemAtPosition(position);
			
			AdnMaterial material = (AdnMaterial)item;
			
			ActivityExchange.SelectedMaterial = material;
			
			Intent intent = new Intent(_activity, AdnMaterialPropsActivity.class);
			startActivity(intent); 
		}
	};
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	public void OnWebServiceSuccess(String JsonMsg) 
	{
		final Activity activity = this;
		
		try 
		{
			Type collectionType = new TypeToken<AdnMaterial[]>(){}.getType();
			
			final AdnMaterial[] materials = (AdnMaterial[]) 
				_parser.fromJson(JsonMsg , collectionType);
			
			this.runOnUiThread(new Runnable() 
			{
				public void run() 
				{	    
					AdnMaterialsItemAdapter adapter = new AdnMaterialsItemAdapter(
						activity, 
						R.layout.materials_listview_item, 
						materials);
					
					ListView lv = (ListView) findViewById(R.id.materialsList);
					
					lv.setAdapter(adapter);
					lv.setOnItemClickListener(_itemClickListener);
					
					setTitle("ADN Material Demo");
					activity.setProgressBarIndeterminateVisibility(false);
				}
			});	
		}
		catch (Exception ex) 
		{
			OnWebServiceFailed(activity, ex);
			return;
		}
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	public void OnWebServiceFailed(Exception ex) 
	{
		OnWebServiceFailed(this, ex);
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	private void OnWebServiceFailed(Activity activity, Exception ex)
	{
		final Activity f_activity = activity;
		
		f_activity.runOnUiThread(new Runnable() 
		{
			public void run() 
			{	 
				Toast.makeText(f_activity, "Web service call failed..." , Toast.LENGTH_SHORT).show();
				
				f_activity.setTitle("ADN Material Demo");
				f_activity.setProgressBarIndeterminateVisibility(false);
			}
		});
	}
}





