package Autodesk.ADN.Material;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Configuration;
import android.os.Bundle;
import android.text.Editable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

import com.google.gson.Gson;

//////////////////////////////////////////////////////////////////////////////////////////
// This class handles the material properties display and modification
//
//////////////////////////////////////////////////////////////////////////////////////////
public class AdnMaterialPropsActivity 
	extends Activity
{
	//////////////////////////////////////////////////////////////////////////////////////
	// Members
	//
	//////////////////////////////////////////////////////////////////////////////////////
	Gson _parser;
	
	Activity _activity;
	
	AdnMaterialPropsItemAdapter _adapter;
	
	private static final String URL = "http://adnmaterial.cloudapp.net/MaterialSrv.svc/rest/";
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	private class AdnMaterialPropsItemAdapter 
		extends ArrayAdapter<AdnMaterialProperty>
	{ 	  
		int _resource; 
		Context _context; 
		AdnMaterialProperty _data[] = null;
	
		//////////////////////////////////////////////////////////////////////////////////
		//
		//
		//////////////////////////////////////////////////////////////////////////////////
		public AdnMaterialPropsItemAdapter(
			Context context, 
			int resource,  
			AdnMaterialProperty data[]) 
		{ 
			super(context, resource, data); 
			
			_resource = resource; 
			
			_context = context;
			
			_data = data;
		} 
		
		//////////////////////////////////////////////////////////////////////////////////
		//
		//
		//////////////////////////////////////////////////////////////////////////////////
		@Override
		public View getView(int position, View convertView, ViewGroup parent) 
		{
			View row = convertView;
			
			TagHolder holder;
			
			if(row == null)
			{
				LayoutInflater inflater = ((Activity)_context).getLayoutInflater();
				
				row = inflater.inflate(_resource, parent, false);
				
				holder = new TagHolder();
				
				holder.Image = (ImageView)row.findViewById(R.id.ImageProps);
				holder.Text = (TextView)row.findViewById(R.id.TextProps);
				
				row.setTag(holder);
			}
			else
			{
				holder = (TagHolder)row.getTag();
			}
			
			AdnMaterialProperty data = _data[position];
			
			holder.Text.setText(
				data.Name + ": " + data.Value + (data.Editable ? "" : " (read only)"));
			
			holder.Image.setImageResource(R.drawable.property);
			
			return row;
		}
		
		class TagHolder
		{
			ImageView Image;
			TextView Text;
		}
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	@Override
	protected void onCreate(Bundle savedInstanceState) 
	{
		super.onCreate(savedInstanceState);
		
		_activity = this;
		
		_parser = new Gson();
		
		setTitle("ADN Material Demo");

		setResult(Activity.RESULT_CANCELED);
		setContentView(R.layout.properties_listview);
		
		AdnMaterialProperty[] properties = 
			ActivityExchange.SelectedMaterial.GetProperties();

		_adapter = new AdnMaterialPropsItemAdapter(
			_activity, 
			R.layout.properties_listview_item, 
			properties);
		
		View header = (View)getLayoutInflater().inflate(
				R.layout.properties_listview_header, null);
		
		ListView lv = (ListView) findViewById(R.id.propertiesList);
		
		lv.addHeaderView(header, null, false);
		lv.setAdapter(_adapter);
		lv.setOnItemClickListener(_itemClickListener);
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	@Override
	public void onConfigurationChanged(Configuration newConfig) 
	{
		super.onConfigurationChanged(newConfig);
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	//
	//
	//////////////////////////////////////////////////////////////////////////////////////
	@Override
	protected void onDestroy() 
	{
		super.onDestroy();
	}
	
	//////////////////////////////////////////////////////////////////////////////////////
	// When a list item is clicked we display an AlertDialog to modify the property
	//
	//////////////////////////////////////////////////////////////////////////////////////
	private OnItemClickListener _itemClickListener = new OnItemClickListener() 
	{
		public void onItemClick(AdapterView<?> parent, View v, int position, long id) 
		{
			Object item = parent.getItemAtPosition(position);
			
			final AdnMaterialProperty property = (AdnMaterialProperty)item;
			
			if(!property.Editable)
				return;
			
			AlertDialog.Builder alert = new AlertDialog.Builder(_activity);

        	alert.setTitle("Edit Property...");
        	alert.setMessage(property.Name);

        	// Set an EditText view to get user input 
        	final EditText input = new EditText(_activity);
        	
        	input.setText(property.Value);
        	
        	alert.setView(input);
        	 			
        	// OK button handling code
        	alert.setPositiveButton("Ok", new DialogInterface.OnClickListener()
        	{
            	public void onClick(DialogInterface dialog, int whichButton) 
            	{
            	  Editable value = input.getText();
            	  
            	  property.Value = value.toString();
            	  
              	  _adapter.notifyDataSetChanged();
              	
            	  ActivityExchange.SelectedMaterial.SetProperty(property);
            	  
            	  WebServiceConnector webSrvConnector = new WebServiceConnector(
  	    			URL, 
  	    			"Material");
                      
                  //Perform Web Service POST operation
                  Gson gson = new Gson();
                              
                  String jsonParameters = gson.toJson(ActivityExchange.SelectedMaterial);
                  
                  webSrvConnector.AddRequestParam(jsonParameters);
                  
                  webSrvConnector.InvokeWebPost();
            	}
        	});

        	// Cancel button handling code
        	alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() 
        	{
        		public void onClick(DialogInterface dialog, int whichButton) 
			    {
			        // Canceled.
			    }
        	});

        	alert.show();
		}
	};
}



