//
//  DetailViewController.m
//  AdnMaterialViewer
//
//  Created by Adam Nagy on 12/10/2012.
//  Copyright (c) 2012 Adam Nagy. All rights reserved.
//

#import "DetailViewController.h"

#import "ServerConnection.h"

@interface DetailViewController ()
@property (strong, nonatomic) UIPopoverController * masterPopoverController;
- (void)configureView;
@end

@implementation DetailViewController

- (void)deviceOrientationDidChange:(NSNotification *)notification
{
  [self configureView];
}

#pragma mark - Managing the detail item

- (void)setDetailItem:(id)newDetailItem
{
  if (_detailItem != newDetailItem) {
    _detailItem = newDetailItem;
    
    // Update the view.
    [self configureView];
  }
  
  if (self.masterPopoverController != nil) {
    [self.masterPopoverController dismissPopoverAnimated:YES];
  }
}

- (void)configureView
{
  // Update the user interface for the detail item.
  
  [self.tableView reloadData];
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  
  [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
  [[NSNotificationCenter defaultCenter] addObserver: self
    selector: @selector(deviceOrientationDidChange:)
    name: UIDeviceOrientationDidChangeNotification
    object: nil];
  
	// Do any additional setup after loading the view, typically from a nib.
  [self configureView];
}

- (void)didReceiveMemoryWarning
{
  [super didReceiveMemoryWarning];
  // Dispose of any resources that can be recreated.
}

#pragma mark - Table view

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  if (_detailItem == nil)
    return 0;
  
  return _detailItem.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  UITableViewCell * cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
  
  CGRect rect = self.view.bounds;
  
  UITextField * myTextField = [[UITextField alloc] initWithFrame:CGRectMake(0,10,floorf(rect.size.width / 2),20)];
  myTextField.adjustsFontSizeToFitWidth = NO;
  myTextField.backgroundColor = [UIColor clearColor];
  myTextField.autocorrectionType = UITextAutocorrectionTypeNo;
  myTextField.autocapitalizationType = UITextAutocapitalizationTypeWords;
  myTextField.keyboardType = UIKeyboardTypeDefault;
  myTextField.returnKeyType = UIReturnKeyDone;
  myTextField.clearButtonMode = UITextFieldViewModeNever;
  myTextField.delegate = self;
  
  NSArray * keys = [_detailItem allKeys];
  
  cell.textLabel.text = [keys objectAtIndex:indexPath.section];
  
  myTextField.text = [[_detailItem objectForKey:cell.textLabel.text] description];
  
  cell.accessoryView = myTextField;
  
  return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
  // Return NO if you do not want the specified item to be editable.
  return YES;
}

#pragma mark - Text field

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
  
  [textField resignFirstResponder];
  
  UITableViewCell * cell = (UITableViewCell *)[textField superview];
  NSString * key = cell.textLabel.text;
  
  [_detailItem setValue:textField.text forKey:key];
  
  [ServerConnection setMaterial:_detailItem];
  
  return true;
}

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
  NSString * property = ((UITableViewCell *)textField.superview).textLabel.text;

  if ([property isEqualToString:@"Name"])
    return NO;
  else
    return YES;
}

#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
  barButtonItem.title = NSLocalizedString(@"Materials", @"Materials");
  [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
  self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
  // Called when the view is shown again in the split view, invalidating the button and popover controller.
  [self.navigationItem setLeftBarButtonItem:nil animated:YES];
  self.masterPopoverController = nil;
}

@end
