//
//  MasterViewController.m
//  AdnMaterialViewer
//
//  Created by Adam Nagy on 12/10/2012.
//  Copyright (c) 2012 Adam Nagy. All rights reserved.
//

#import "MasterViewController.h"

#import "DetailViewController.h"
#import "ServerConnection.h"

@interface MasterViewController ()
{
  NSArray * materials;
}
@end

@implementation MasterViewController

- (void)awakeFromNib
{
  if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
      self.clearsSelectionOnViewWillAppear = NO;
      self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
  }
  
  [super awakeFromNib];
}

- (void)viewDidLoad
{
  [super viewDidLoad];

  UIBarButtonItem * refreshButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(refreshTable:)];
  
  self.navigationItem.rightBarButtonItem = refreshButton;
  self.detailViewController = (DetailViewController *)[[self.splitViewController.viewControllers lastObject] topViewController];
  
  materials = [ServerConnection getMaterials];
}

- (void)refreshTable:(id)sender
{
  self.detailViewController.detailItem = nil;
  
  [self.detailViewController.tableView reloadData];
  
  materials = [ServerConnection getMaterials];

  [self.tableView reloadData];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return materials.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];

    NSDictionary * entry = [materials objectAtIndex:indexPath.row];
  
    cell.textLabel.text = [entry objectForKey:@"Name"];
  
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
        NSDictionary * entry = [materials objectAtIndex:indexPath.row];
        self.detailViewController.detailItem = entry;
    }
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"showDetail"]) {
        NSIndexPath *indexPath = [self.tableView indexPathForSelectedRow];
        NSDictionary * entry = [materials objectAtIndex:indexPath.row];
        [[segue destinationViewController] setDetailItem:entry];
    }
}

@end
