﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MaterialWebRole;
using Newtonsoft.Json;
using System.Net;
using System.IO;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Serialization.Json;

namespace AdnMaterialClient
{
    public partial class MaterialClient : Form
    {
        /////////////////////////////////////////////////////////////////////////////
        // Service client for SOAP EndPoint
        //
        /////////////////////////////////////////////////////////////////////////////
        MaterialSrvClient _serviceClient;

        /////////////////////////////////////////////////////////////////////////////
        // Service client for SOAP EndPoint
        //
        /////////////////////////////////////////////////////////////////////////////
        string HostAddress = "adnmaterial.cloudapp.net";

        /////////////////////////////////////////////////////////////////////////////
        // Constructor
        //
        /////////////////////////////////////////////////////////////////////////////
        public MaterialClient()
        {
            InitializeComponent();

            BeginRefreshMaterialsSoap();

            _mainTabCtrl.SelectedIndexChanged += 
                new EventHandler(_mainTabCtrl_SelectedIndexChanged);
        }

        void _mainTabCtrl_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (_mainTabCtrl.SelectedIndex)
            {
                case 0:

                    BeginRefreshMaterialsSoap();
                    break;

                case 1:

                    BeginRefreshMaterialsRest();
                    break;

                default:
                    break;
            }
        }

        /////////////////////////////////////////////////////////////////////////////
        // GetMaterials Async SOAP
        //
        /////////////////////////////////////////////////////////////////////////////
        void BeginRefreshMaterialsSoap()
        {
            Cursor = Cursors.WaitCursor;

            _serviceClient = new MaterialSrvClient("WCFSoapEndpoint");

            IAsyncResult asyncRes = _serviceClient.BeginGetMaterials(
                GetMaterialsAsyncInfoResponderSoap,
                null);
        }

        delegate void RefreshMaterialsDelegateSoap(AdnMaterial[] materials);

        void GetMaterialsAsyncInfoResponderSoap(IAsyncResult result)
        {
            try
            {
                AdnMaterial[] materials = _serviceClient.EndGetMaterials(result);

                if (result.IsCompleted)
                {
                    this.Invoke(
                        new RefreshMaterialsDelegateSoap(EndRefreshMaterialsSoap),
                        new object[] { materials });
                }
            }
            catch(Exception ex)
            {
                //System.Windows.Forms.MessageBox.Show("Soap: " + ex.Message);
            }
        }

        void EndRefreshMaterialsSoap(AdnMaterial[] materials)
        {
            _materialCtrlSoap.LoadMaterials(materials);

            Cursor = Cursors.Default;
        }

        private void bRefreshSoap_Click(object sender, EventArgs e)
        {
            BeginRefreshMaterialsSoap();
        }

        /////////////////////////////////////////////////////////////////////////////
        // SetMaterial Async SOAP
        //
        /////////////////////////////////////////////////////////////////////////////
        private int _nbMaterialsUploaded = 0;
        
        void BeginSetMaterialSoap(AdnMaterial material)
        {
            Cursor = Cursors.WaitCursor;

            IAsyncResult asyncRes = _serviceClient.BeginPostMaterial(material,
                SetMaterialAsyncInfoResponder,
                null);
        }

        void SetMaterialAsyncInfoResponder(IAsyncResult result)
        {
            try
            {
                bool res = _serviceClient.EndPostMaterial(result);

                this.Invoke(
                    new UploadDoneDelegateSoap(UploadDoneRest),
                    null);
            }
            catch (Exception ex)
            {
                //System.Windows.Forms.MessageBox.Show("Rest:" + ex.Message);
            }
        }

        private void bCommitSoap_Click(object sender, EventArgs e)
        {
            AdnMaterial[] materials = _materialCtrlSoap.GetMaterials();

            _nbMaterialsUploaded = materials.Length;

            foreach (AdnMaterial material in materials)
            { 
                BeginSetMaterialSoap(material);
            }
        }

        delegate void UploadDoneDelegateSoap();

        void UploadDoneSoap()
        {
            if(_nbMaterialsUploaded == 0)
                Cursor = Cursors.Default;
        }

        /////////////////////////////////////////////////////////////////////////////
        // GetMaterials Async REST
        //
        /////////////////////////////////////////////////////////////////////////////
        private void BeginRefreshMaterialsRest()
        {
            Cursor = Cursors.WaitCursor;

            HttpWebRequest request = WebRequest.Create(
                "http://" + HostAddress + "/MaterialSrv.svc/rest/Materials")
                    as HttpWebRequest;

            request.BeginGetResponse(
                new AsyncCallback(GetMaterialsAsyncInfoResponderRest),
                request);
        }

        private delegate void RefreshMaterialsDelegateRest(AdnMaterial[] materials);

        private void GetMaterialsAsyncInfoResponderRest(IAsyncResult result)
        {
            try
            {
                using (HttpWebResponse response =
                    (result.AsyncState as HttpWebRequest).EndGetResponse(result)
                        as HttpWebResponse)
                {
                    if (result.IsCompleted)
                    {
                        StreamReader reader = new StreamReader(response.GetResponseStream());

                        string jsonMsg = reader.ReadToEnd();

                        var materials = JsonConvert.DeserializeObject<List<AdnMaterial>>(jsonMsg);

                        this.Invoke(
                            new RefreshMaterialsDelegateRest(EndRefreshMaterialsRest),
                            new object[] { materials.ToArray() });
                    }
                }
            }
            catch (Exception ex)
            {
                //System.Windows.Forms.MessageBox.Show("Rest: " + ex.Message);
            }
        }

        private void EndRefreshMaterialsRest(AdnMaterial[] materials)
        {
            _materialCtrlRest.LoadMaterials(materials);

            Cursor = Cursors.Default;
        }

        private void bRefreshRest_Click(object sender, EventArgs e)
        {
            BeginRefreshMaterialsRest();
        }

        /////////////////////////////////////////////////////////////////////////////
        // GetMaterials Async REST
        //
        /////////////////////////////////////////////////////////////////////////////
        private void bCommitRest_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;

            AsyncDelegate caller = new AsyncDelegate(DoPostRequest);

            IAsyncResult result = caller.BeginInvoke(
                _materialCtrlRest.GetMaterials(),
                new AsyncCallback(Callback), 
                null);
        }

        public delegate bool AsyncDelegate(AdnMaterial[] materials);

        bool DoPostRequest(AdnMaterial[] materials)
        {
            try
            {
                string url =
                "http://" + HostAddress + "/MaterialSrv.svc/rest/Material";

                foreach (AdnMaterial material in materials)
                {
                    string jsonMsg = JsonConvert.SerializeObject(material,
                        Formatting.None,
                        new JsonSerializerSettings { });

                    WebClient wc = new WebClient();
                    
                    wc.Headers["Content-type"] = "application/json";
                    wc.Encoding = Encoding.UTF8;

                    string res = wc.UploadString(new Uri(url), "POST", jsonMsg);
                }

                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        void Callback(IAsyncResult result)
        {
            AsyncResult asyncResult = (AsyncResult)result;

            AsyncDelegate caller = (AsyncDelegate)asyncResult.AsyncDelegate;

            bool res = caller.EndInvoke(result);

            this.Invoke(
                new UploadDoneDelegateRest(UploadDoneRest),
                null);
        }

        delegate void UploadDoneDelegateRest();

        void UploadDoneRest()
        {
            Cursor = Cursors.Default;
        }
    }
}
