﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MaterialWebRole;

namespace AdnMaterialClient
{
    public partial class MaterialCtrl : UserControl
    {
        /////////////////////////////////////////////////////////////////////////////
        // Constructor
        //
        /////////////////////////////////////////////////////////////////////////////
        public MaterialCtrl()
        {
            InitializeComponent();

            _materialList.EditableControls.Add(_tbEdit);
            _materialList.EditableControls.Add(_tbEdit);
            _materialList.EditableControls.Add(_tbEdit);

            _materialList.SubItemClicked +=
               new ListViewEditableCtrls.SubItemEventHandler(SubItemClicked);

            _materialList.SubItemEndEditing +=
                new ListViewEditableCtrls.SubItemEndEditingEventHandler(
                    SubItemEndEditing);
        }

        /////////////////////////////////////////////////////////////////////////////
        // Fiils up control with materials
        //
        /////////////////////////////////////////////////////////////////////////////
        public void LoadMaterials(AdnMaterial[] materials)
        {
            _materialList.Items.Clear();

            foreach (AdnMaterial material in materials)
            { 
                ListViewItem item = _materialList.Items.Add(material.Name);

                item.Tag = material;

                item.SubItems.Add(material.Price.ToString());
                item.SubItems.Add(material.Manufacturer);
            }
        }

        /////////////////////////////////////////////////////////////////////////////
        // Returns list of materials
        //
        /////////////////////////////////////////////////////////////////////////////
        public AdnMaterial[] GetMaterials()
        {
            List<AdnMaterial> materials = new List<AdnMaterial>();

            foreach (ListViewItem item in _materialList.Items)
            {
                materials.Add(item.Tag as AdnMaterial);
            }

            return materials.ToArray();
        }

        /////////////////////////////////////////////////////////////////////////////
        // ListView subitem clicked event
        //
        /////////////////////////////////////////////////////////////////////////////
        void SubItemClicked(
           object sender,
           ListViewEditableCtrls.SubItemEventArgs e)
        {
            //Prevents Material Name form being edited
            if(e.SubItem != 0)
                _materialList.StartEditing(e.Item, e.SubItem);
        }

        /////////////////////////////////////////////////////////////////////////////
        // Subitem end editing event
        //
        /////////////////////////////////////////////////////////////////////////////
        void SubItemEndEditing(
            object sender,
            ListViewEditableCtrls.SubItemEndEditingEventArgs e)
        {
            AdnMaterial material = e.Item.Tag as AdnMaterial;

            switch (e.SubItem)
            {
                case 1:
                    double price = 0.0;

                    if(double.TryParse(e.Item.SubItems[e.SubItem].Text, out price))
                        material.Price = price;

                    break;

                case 2:
                    material.Manufacturer = e.Item.SubItems[e.SubItem].Text;
                    break;

                default:
                    break;
            }
        }
    }
}
