﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;

namespace MaterialWebRole
{
    /////////////////////////////////////////////////////////////////////////////
    // Definition of our DataContract, this is the data that will transit 
    // through our web service
    //
    /////////////////////////////////////////////////////////////////////////////
    [DataContract]
    public class AdnMaterial
    {
        public AdnMaterial(
            string name,
            double price,
            string manufacturer)
        {
            Name = name;

            Price = price;

            Manufacturer = manufacturer;
        }

        [DataMember]
        public string Name
        {
            get;
            set;
        }

        [DataMember]
        public double Price
        {
            get;
            set;
        }

        [DataMember]
        public string Manufacturer
        {
            get;
            set;
        }
    }

    /////////////////////////////////////////////////////////////////////////////
    // Definition of the ServiceContract, this is what our service is capable of
    //
    /////////////////////////////////////////////////////////////////////////////
    [ServiceContract]
    public interface IMaterialSrv
    {
        [OperationContract]
        [WebInvoke(
            Method = "GET",
            UriTemplate = "/Materials",
            ResponseFormat = WebMessageFormat.Json)]
        AdnMaterial[] GetMaterials();

        [OperationContract]
        [WebInvoke(
            Method = "GET",
            UriTemplate = "/Material/{materialName}",
            ResponseFormat = WebMessageFormat.Json)]
        AdnMaterial GetMaterial(
            [MessageParameter(Name = "materialName")]
            string materialName);

        [OperationContract]
        [WebInvoke(
            Method = "POST",
            UriTemplate = "/Material",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json)]
        bool PostMaterial(
            [MessageParameter(Name = "material")]
            AdnMaterial material);

        [OperationContract]
        [WebInvoke(
            Method = "DELETE",
            UriTemplate = "/Material/{materialName}",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json)]
        bool DeleteMaterial(
            [MessageParameter(Name = "materialName")]
            string materialName);
    }
}
